; table_bidirectional_dec10_wi_ale.A51 	store ADC data in table, & play it out to DAC:
;  let a switch determine direction of playback (forward/reverse) (11/09)
;  add table-end definitions at program head (12/10)
;  TIMED BY INTERRUPT	
;  Uses @R_n addressing mode rather than second DPTR; replace XCH with PUSH (4/05)
;  modified 4/08 by disabling interrupts till sample handled, to allow single-stepping with fast interrupts

    $NOSYMBOLS        ; keeps listing short

; default Ride7 path, Mar. 16
    $INCLUDE (C:\Program Files\Raisonance\Ride\inc\51\Dallas\REG320.INC) 
    $INCLUDE (C:\Program Files\Raisonance\Ride\inc\51\Dallas\vectors320.INC) 

;	REGISTER USE: end-address: R7,R6; 
;  assumes switch at P3.4 to select direction of playback

	SAMPLEFLAG	EQU 0h	; first bit-addressable, in scratchpad RAM: used to let ISR talk to MAIN, saying 'take sample'
	UPFLAG EQU 1h			; bit that determines playback direction: 1 for forward, 0 for reverse
	
	STACKSTART EQU 07Fh
	
	; for playback-direction control (controlled by P3.4)
	FORWARD EQU 00h		; value for DPS register: zero in bit 6 says INC means INC
	REVERSE EQU 40h		; value for DPS register: one in bit 6 says INC means DEC
	
	END_HI EQU 80h		; hi byte of end address
	END_LO EQU 00H		; low byte of end address
	
	ORG 0
	
 	AJMP START
	
	ORG 140h
	
START:MOV 0C4h, #04h	; turn on ALE during ROM run (bit 2, ALEON, of PMR register, C4h)
	   MOV SP, #STACKSTART
		MOV R7, #END_HI		; set reference address (end of table: last address below I/O space)
		MOV R6, #END_LO		;  ...this is the low byte--tiny table, for testing
		
		
STARTUP: ACALL INITPTR	; INITIALIZE ptrs for ADC & DAC data, etc.
		MOV DPTR, #800h		; init main pointer to first writeable location
		ACALL OKINT		; go enable interrupts
		
FILLTABLE:	SETB EA				; re-enable interrupts
		JNB SAMPLEFLAG,FILLTABLE	; hang here till it's time to take a sample
		CLR EA				; disable interrupts till we've taken care of the task
		CLR SAMPLEFLAG				; clear the flag set by ISR
		
		ACALL GETSAMPLE

		MOVX @DPTR, A			; store the sample
		INC DPTR

		ACALL COMP_PTR			; see if we've hit end address
		JC FILLTABLE			; ...if not, do it again (c ==> movx ptr < ref)
		
		
REINIT: JNB P3.4, RVRS		; test switch to determine whether to playback forward or in revers
		SETB UPFLAG				; make soft flag match switch setting (prevents confusion if switch
		SJMP PROCEED			;  is flipped during playback)
		
RVRS:	CLR UPFLAG				

PROCEED:	JNB UPFLAG, DOWN	; test soft flag
UPINIT:		MOV DPTR, #800h	; if UP, then start at bottom of table
		MOV DPS, #FORWARD		; and set direction-bit for DPTR INC
		SJMP PLAYBACK
		
DOWN: 		MOV DPH,R7				; recall end-of-table address
		MOV DPL,R6
		MOV DPS, #REVERSE		; if DOWN, leave DPTR at final value and set DPTR INC to mean "DEC"
		INC DPTR					; throw away highest address: nothing was written there
	
PLAYBACK:			SETB EA				; re-enable interrupts
		JNB SAMPLEFLAG,PLAYBACK	; hang here till it's time to PLAY a sample
		CLR EA				; disable interrupts till we've taken care of the task
		CLR SAMPLEFLAG				; clear the flag set by ISR
		
		MOVX A,@DPTR			; get the sample from table, not new from ADC
		MOVX @R0,A				; send sample out to DAC
		INC DPTR					; advance the table pointer
		ACALL FANCY_COMP_PTR			; see if we've hit end address

		JC PLAYBACK			; ...if not, do it again
		SJMP REINIT				; otherwise, reinit and play again
		
; ----SUBROUTINES--

INITPTR:	MOV P2, #80h		; set I/O 'base' address: high byte of port address (fixed)
		MOV R1, #03h			; init low-byte to point to ADC
		MOV R0, #02h			; init low-byte to point to DAC
		CLR SAMPLEFLAG			; clear flag bit that ISR uses as signal to Main
		RET 	

GETSAMPLE:  MOVX A, @R1		; Read ADC
	MOVX @R1, A	; send anything--just a pulse to START of ADC
	RET
	
	
; FANCY END TEST: tests for 800h if REVERSE, tests for end address if FORWARD	
FANCY_COMP_PTR:	JNB UPFLAG, STRANGE	; if not UP, test for start, not end
	ACALL COMP_PTR
	SJMP EXIT
	
STRANGE: MOV R3, DPH		; if not UP, test for start address, 800h, minus one
								;  (because pointer is decremented before test)
	CJNE R3, #07h, NOT_DONE
	MOV R3, DPL
	CJNE R3, #0FFh, NOT_DONE		; if high-byte matches, look at low byte
	CLR C						; if we land here, we've reached start address
	SJMP EXIT            ;  clear CY flag, to tell main program to reinitialize
	
NOT_DONE:	SETB C					; set CY flag, used by main program to indicate 'not done'
EXIT:	RET
	
; --POINTER COMPARE: THIS ONE IS ASSIGNED AN ADDRESS, BECAUSE IT MAY BE USEFUL MORE GENERALLY: 
;  callable from other programs: this subroutine alters C flag. C means moving pointer less than reference

	ORG 200h

COMP_PTR:	PUSH ACC		; save A, which will be messed up
		CLR C
		MOV A, DPL
		SUBB A, R6			; see if ptr less than ref (low byte)
		MOV A, DPH
		SUBB A, R7			; after this, CY flag holds answer: true if ref > ptr
		POP ACC			; recover old A value
		RET
		
; ----NOW ENABLE INTERRUPTS----
OKINT:	
	SETB TCON.0		; make INT0 Edge-sensitive	
	SETB IE.0		; enable INT0
	RET
		
		
;--Interrupt Response: ISR----------
		ORG 03h				; this is INT0 vector address		
		SETB SAMPLEFLAG	; set flag bit, to inform Main
		RETI
		
	END
	
	 
